import std/[json, options, sequtils, strutils, times]
import db_connector/db_sqlite
import waterpark/sqlite
import fiber_orm, timeutils, uuids

export fiber_orm.NotFoundError
export fiber_orm.PaginationParams
export fiber_orm.PagedRecords
export fiber_orm.enableDbLogging
export sqlite.close

import ./[logging, models]

type
  WdiwtltDb* = SqlitePool


func toJsonHook*(dt: DateTime): JsonNode = %(dt.formatIso8601)
proc fromJsonHook*(dt: var DateTime, n: JsonNode): void =
  dt = n.getStr.parseIso8601


proc initDB*(dbPath: string): SqlitePool =
  newSqlitePool(10, dbPath)

generateProcsForModels(WdiwtltDb,
  [Artist, Album, MediaFile, Tag, Playlist, Bookmark, Image])

generateJoinTableLookups(WdiwtltDb, Artist, Album, "artists_albums")
generateJoinTableLookups(WdiwtltDb, Artist, MediaFile, "artists_media_files")
generateJoinTableLookups(WdiwtltDb, Album, MediaFile, "albums_media_files")
generateJoinTableLookups(WdiwtltDb, Playlist, MediaFile, "playlists_media_files")
generateJoinTableLookups(WdiwtltDb, MediaFile, Tag, "media_files_tags")
generateJoinTableLookups(WdiwtltDb, Artist, Image, "artists_images")
generateJoinTableLookups(WdiwtltDb, Album, Image, "albums_images")

proc removeEmptyAlbums*(sql: WdiwtltDb) =
  ## Remove albums that have no media files.

  let query = """
      DELETE FROM albums WHERE id IN (
          SELECT DISTINCT al.id
          FROM albums al LEFT OUTER JOIN albums_media_files almf ON
              al.id = almf.album_id
          WHERE almf.album_id IS NULL)"""

  getLogger("wdiwtlt/db").debug(%*{
    "msg": "Deleting empty albums.",
    "query": query})

proc removeEmptyArtists*(sql: WdiwtltDb) =
  ## Remove artists that have no albums.

  let query = """
      DELETE FROM artists WHERE id IN (
          SELECT DISTINCT ar.id
          FROM artists ar LEFT OUTER JOIN artists_albums aral ON
              ar.id = aral.artist_id
          WHERE aral.artist_id IS NULL)"""

  getLogger("wdiwtlt/db").debug(%*{
    "msg": "Deleting empty artists.",
    "query": query})

proc removeEmptyPlaylists*(sql: WdiwtltDb) =
  ## Remove playlists that have no media files.

  let query = """
      DELETE FROM playlists WHERE id IN (
          SELECT DISTINCT pl.id
          FROM playlists pl LEFT OUTER JOIN playlists_media_files plmf ON
              pl.id = plmf.playlist_id
          WHERE plmf.playlist_id IS NULL)"""

  getLogger("wdiwtlt/db").debug(%*{
    "msg": "Deleting empty playlists.",
    "query": query})
